#!/usr/bin/env python3

import argparse
from typing import IO

from alphatools.utils.alphatools_prompt import AlphaToolsCliPrompt
from alphatools.utils.token_manager import TokenManager


class InstrumentsCliPrompt(AlphaToolsCliPrompt):
    def __init__(self, completekey: str = "tab", stdin: IO[str] | None = None, stdout: IO[str] | None = None) -> None:
        super().__init__(completekey, stdin, stdout)
        self.token_manager = TokenManager()


    def do_getInstrumentInfo(self, instrument_id):
        print(self.token_manager.get_instrument(int(instrument_id)))
        pass

    def do_getInstrumentInfoForSymbol(self, symbol):
        print(self.token_manager.get_instrument(symbol))

    def help_getInstrumentInfo(self):
        print("Returns Instrument symbol for instrument id")

    def help_getInstrumentInfoForSymbol(self):
        print("Returns Instrument symbol for symbol")


if __name__ == '__main__':
    parser = argparse.ArgumentParser()

    parser.add_argument("-c", "--config_file",
                        help="Config File with credentials", required=False, default='/Users/jaskiratsingh/projects/smart-api-creds.ini')


    InstrumentsCliPrompt().cmdloop()


import json
import logging

logging.basicConfig(
    format='[%(asctime)s] %(levelname)-8s {%(filename)s:%(lineno)d} : %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S', level=logging.WARNING
)

def cache_to_file(dir_name, _hash):
    def decorator(original_func):
        logger = logging.getLogger(__name__)

        def new_func(self, params, force_reload = False):
            hash_result = _hash(params)
            file_name = dir_name + "/{}.csv".format(hash_result)
            logger.debug("Reading data from file: {}. Force reload set to {}".format(file_name, force_reload))
            try:
                f = open(file_name, 'r')
                cache = json.load(f)
                f.close()
            except (IOError, ValueError):
                cache = {}

            if hash_result not in cache or not cache[hash_result] or force_reload:
                logger.debug("Could not find results in cache. Calling the function to get results")
                cache[hash_result] = original_func(self, params)
                logger.debug("Saving data to file: {}".format(file_name))
                f = open(file_name, 'w')
                json.dump(cache, f)
                f.close()
            else:
                logger.debug("Results already exist. Fetching from cache")
            return cache[hash_result]

        return new_func

    return decorator



import datetime
from unittest import TestCase
from alphatools import backtesting_app
from datetime import datetime

from alphatools.utils.token_manager import TokenManager
import logging

logging.basicConfig(level=logging.INFO)


class BackTestingAppOnDummyData(backtesting_app.BackTestingApp):
    dummy_feed = [
        ['2022-12-20T11:39:00+05:30', 973.0, 973.0, 972.25, 972.25, 40],
        ['2022-12-20T11:40:00+05:30', 972.25, 972.25, 972.25, 972.25, 1],
        ['2022-12-20T11:41:00+05:30', 972.25, 972.25, 972.25, 972.25, 1],
        ['2022-12-20T11:42:00+05:30', 974.2, 974.2, 972.6, 972.6, 11],
        ['2022-12-20T11:43:00+05:30', 972.6, 972.6, 972.6, 972.6, 1],
        ['2022-12-20T11:44:00+05:30', 972.6, 974.35, 972.6, 974.35, 12],
        ['2022-12-20T11:45:00+05:30', 972.5, 974.55, 972.5, 974.55, 19],
        ['2022-12-20T11:46:00+05:30', 974.7, 974.95, 974.05, 974.95, 20],
        ['2022-12-20T11:47:00+05:30', 972.45, 972.45, 972.0, 972.0, 124],
        ['2022-12-20T11:48:00+05:30', 973.7, 973.7, 972.05, 972.05, 18],
        ['2022-12-20T11:49:00+05:30', 972.05, 972.05, 972.05, 972.05, 1]
    ]

    def _get_candle_info_results(self, _date, token, exchange):
        return self.dummy_feed

    def on_md(self, data_row):
        self.trade(data_row.Token, 1)

    def post_simulation(self):
        self.logger.info("Total Pnl: {}".format(self.get_total_pnl()))


class TestBackTestingApp(TestCase):
    logger = logging.getLogger(__name__)
    def setUp(self) -> None:
        self.app = BackTestingAppOnDummyData("/Users/jaskiratsingh/projects/smart-api-creds.ini")
        self.app.add_instrument(53825, "NFO")
        self.app.add_instrument(48756, "NFO")
        self.app.set_start_date(datetime.strptime('2022-12-20 11:39:00+05:30', '%Y-%m-%d %H:%M:%S%z'))
        self.app.set_end_date(datetime.strptime('2022-12-20 11:39:00+05:30', '%Y-%m-%d %H:%M:%S%z'))
        self.app.set_interval('ONE_MINUTE')

    def test_run_backtesting_engine_basic(self):
        self.app.load_data()                     # Loads the data into a dataframe
        self.app.get_row_number(datetime.strptime('2022-12-23 11:39:00+05:30', '%Y-%m-%d %H:%M:%S%z'))
        df = self.app.get_candle_info_df()            # Returns the entire simulation dataframe

        assert len(df) == 2 * len(self.app.dummy_feed)

    def test_run_backtesting_engine_advanced(self):
        self.app.load_data()                             # Loads the data into a dataframe
        df1 = self.app.get_candle_info_df(-1)            # Returns entire candle info dataframe
        df2 = self.app.get_candle_info_df(2)             # Returns first two rows of the dataframe
        assert len(df1) == 2 * len(self.app.dummy_feed)
        assert len(df2) == 2

    def test_run_token_manager(self):
        app = TokenManager()
        self.logger.info(app.get_instrument('BANKNIFTY23FEB23FUT'))
        self.logger.info(app.get_instrument(48756))
        self.logger.info(app.get_opt("NIFTY", "09FEB23", 16000, "PE"))




#!/usr/bin/env python3

import argparse
from datetime import datetime

from alphatools.backtesting_app import BackTestingApp

from alphatools.utils import instruments

import logging


class PriceArchiverStrat(BackTestingApp):
    def __init__(self, creds):
        # self.logger.addHandler(handler)
        self.logger.setLevel(logging.DEBUG)
        super().__init__(creds)

    def load_instruments(self, underliers, start_date, end_date):
        self.set_start_date(start_date)
        self.set_end_date(end_date)
        for underlying in underliers:
            nifty_token_list = instruments.get_derivatives(underlying)
            for token_info in nifty_token_list:
                token = token_info['token']
                exch_seg = token_info['exch_seg']
                self.add_instrument(token, exch_seg)

        self.load_data()


if __name__ == '__main__':
    parser = argparse.ArgumentParser()

    parser.add_argument("-u", "--underlying", nargs='+',
                        help="Instrument whose derivatives have to be recorded", required=True)
    parser.add_argument("-s", "--start_date",
                        help="Instrument whose derivatives have to be recorded", required=False,
                        default=datetime.now().strftime('%Y%m%d'))
    parser.add_argument("-e", "--end_date",
                        help="Instrument whose derivatives have to be recorded", required=False,
                        default=datetime.now().strftime('%Y%m%d'))
    args = parser.parse_args()
    start_date = datetime.strptime(args.start_date, '%Y%m%d')
    end_date = datetime.strptime(args.end_date, '%Y%m%d')
    strat = PriceArchiverStrat('/Users/jaskiratsingh/projects/smart-api-creds.papa.ini')
    strat.load_instruments(args.underlying, start_date, end_date)



import logging
import pandas as pd


class InstrumentPnlCalculator:
    logger = logging.getLogger(__name__)
    last_feed_info = pd.Series([], dtype='float64')

    def __init__(self, token):
        self.total_buy_qty = 0
        self.total_sell_qty = 0
        self.total_buy_val = 0
        self.total_sell_val = 0
        self.token = token

    def on_md(self, feed):
        self.last_feed_info = feed
        self.logger.debug("Feed Received by Pnl Calculator: {}".format(self.last_feed_info))

    def trade(self, qty):
        last_price = self.last_feed_info.CLOSE
        if qty > 0:
            self.total_buy_qty += qty
            self.total_buy_val += qty * last_price
        else:
            self.total_sell_qty -= qty
            self.total_sell_val -= qty * last_price

    def get_total_pnl(self):
        position = self.total_buy_qty - self.total_sell_qty
        if not len(self.last_feed_info) :
            return 0
        last_price = self.last_feed_info.CLOSE
        pnl = position * last_price - (self.total_buy_val - self.total_sell_val)
        self.logger.info("Pnl for instrument: {} = {}".format(self.token, pnl))
        return pnl



from alphatools.utils.instrument_pnl_calculator import InstrumentPnlCalculator


class PnlCalculator:
    def __init__(self):
        self.instrument_pnl_calculators = {}

    def on_md(self, token, feed):
        if token in self.instrument_pnl_calculators:
            pnl_calculator = self.instrument_pnl_calculators[token]
            pnl_calculator.on_md(feed)

    def add_instrument_to_watch_list(self, token):
        self.instrument_pnl_calculators[token] = InstrumentPnlCalculator(token)

    def trade(self, token, qty):
        """

        :param token: Token of the instrument to be traded
        :param qty: Quantity traded, positive if bought, negative if sold
        :return:
        """
        instrument_pnl_calc = self.instrument_pnl_calculators[token]
        instrument_pnl_calc.trade(qty)

    def get_total_pnl(self):
        result = 0
        for pnl_calculator in self.instrument_pnl_calculators.values():
            result += pnl_calculator.get_total_pnl()

        return result



import setuptools
import glob


with open("requirements.txt", "r") as fh:
    install_requires = fh.read()

with open("README.md", "r") as fh:
    long_description = fh.read()

__version__ = "3.4.0"

setuptools.setup(
    name="alphatools_jv",
    version=__version__,
    author="Jaskirat Singh",
    author_email="jaskiratsingh1208@gmail.com",
    description="Wrapper over angel broking smartapi to read cached options market data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jaskirat1208/backtest-platform",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    test_suite="test",
    scripts=glob.glob('scripts/*'),
    install_requires=install_requires
)



from cmd import Cmd
 
class AlphaToolsCliPrompt(Cmd):
    def do_exit(self, inp):
        print("Bye")
        return True
     
    def help_exit(self):
        print('exit the application. Shorthand: x q Ctrl-D.')
        
    do_EOF = do_exit
    help_EOF = help_exit


if __name__ == '__main__':
    AlphaToolsCliPrompt().cmdloop()


#!/usr/bin/env python3


import configparser
from typing import IO

from alphatools.utils import smartapi_helper

import argparse
from datetime import datetime
from alphatools.utils.alphatools_prompt import AlphaToolsCliPrompt


def query(args):
    pass    

class SmartApiQueryPrompt(AlphaToolsCliPrompt):
    def __init__(self, args) -> None:
        super().__init__()
        cfg_parser = configparser.ConfigParser()
        cfg_parser.read(args.config_file)
        api_key = cfg_parser.get('SMARTAPI_LOGIN', 'API_KEY')
        client_code = cfg_parser.get('SMARTAPI_LOGIN', 'CLIENT_CODE')
        password = cfg_parser.get('SMARTAPI_LOGIN', 'PASSWORD')
        totp_key = cfg_parser.get('SMARTAPI_LOGIN', 'TOTP_KEY')

        self.api_helper = smartapi_helper.SmartApiHelper(api_key=api_key, client_code=client_code, password=password, totp_key=totp_key)

    def do_getCandleInfo(self, args):
        # print(args.split())
        parser = argparse.ArgumentParser(exit_on_error=False)

        parser.add_argument("-e", "--exchange",
                            help="Config File with credentials", required=False, default='NSE')
        parser.add_argument("-t", "--token",
                            help="Config File with credentials", required=False, default='26000')
        parser.add_argument("-i", "--interval",
                            help="Config File with credentials", required=False, default='ONE_MINUTE')
        parser.add_argument("-d", "--date", help="Date", required=False, default=datetime.today().strftime('%Y%m%d'))
        
        args = parser.parse_args(args.split())
        print(args)
        # exchange, token, interval, date = args.split()
        self._getCandleInfo(args.exchange, args.token, args.interval, args.date) 
    
    def _getCandleInfo(self, exchange, token, interval='ONE_MINUTE', date=datetime.today().strftime('%Y%m%d')):
        _date = datetime.strptime(date, '%Y%m%d')
        candle_info_params = candle_info_params = {
                "exchange": exchange,
                "symboltoken": token,
                "interval": interval,
                "fromdate": datetime.strftime(_date, '%Y-%m-%d 00:00'),
                "todate": datetime.strftime(_date, '%Y-%m-%d 23:59')
            }
        print(self.api_helper.get_candle_info(candle_info_params))


    def help_getCandleInfo(self):
        print("Returns candle info")


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    import sys
    print(sys.argv[1:])
    parser.add_argument("-c", "--config_file",
                        help="Config File with credentials", required=False, default='/Users/jaskiratsingh/projects/smart-api-creds.ini')
    
    args = parser.parse_args()
    
    prompt = SmartApiQueryPrompt(args)
    prompt.cmdloop()


import nsepython
from datetime import datetime
from functools import lru_cache
from enum import Enum
import pandas as pd

from alphatools.utils.token_manager import TokenManager

class ExpiryType(Enum):
    WEEKLY = 0
    MONTHLY = 1


def get_kth_expiry(symbol, k=1, type=ExpiryType.WEEKLY):
    expiry_df = _get_expiry_list(symbol, 'pandas')
    if k <= 0 or k > len(expiry_df):
        raise Exception("Invalid expiry date")

    convert_to_date = lambda exp_date: datetime.strptime(exp_date, '%d-%b-%Y')
    date_list = expiry_df['Date'].apply(convert_to_date)
    if type == ExpiryType.WEEKLY:
        return date_list[0].strftime('%d%b%y').upper()

    last_date = date_list[0]
    for exp_date in date_list:
        if exp_date.month != datetime.now().month:
            return last_date.strftime('%d%b%y').upper()

        last_date = exp_date

    return last_date.strftime('%d%b%y').upper()

@lru_cache(maxsize=None)
def _get_expiry_list(symbol, type='list'):
    try:
        _get_expiry_lists(symbol=symbol, type=type)
    except Exception as e:
        print("Some exception occured. Returning current expiry as list")
        next_expiry = '20-JUL-2023'
        next2next_expiry = '20-JUL-2023'
        if type == 'list':
            convert_to_date = lambda exp_date: _convert_date(exp_date, '%d-%b-%Y', '%d%b%y').upper()
            return [convert_to_date(next_expiry), convert_to_date(next2next_expiry)]
        else:
            return pd.DataFrame({'Date': [next_expiry, next2next_expiry]})

def _get_expiry_lists(symbol, type='list'):
    if type != 'list':
        return nsepython.expiry_list(symbol, type)
    convert_to_date = lambda exp_date: _convert_date(exp_date, '%d-%b-%Y', '%d%b%y').upper()
    date_list = map(convert_to_date, nsepython.expiry_list(symbol, type))
    return list(date_list)


def _convert_date(exp_date, inp_date_fmt, out_date_fmt):
    datetime_obj = datetime.strptime(exp_date, inp_date_fmt)
    return datetime_obj.strftime(out_date_fmt)

def round_to_base(n, base= 10):
    return int(n/base) * base


def get_derivatives(symbol):
    if symbol == "NIFTY" or symbol == 'FINNIFTY':
        min_strike = 18500
        max_strike = 20500
        interval = 50

        token_manager = TokenManager()
        token_list = []
        # Add futures to instrument list
        for exp_date in _get_expiry_list(symbol):
            fut_info = token_manager.get_fut(symbol, exp_date)
            if fut_info != None:
                token_list.append(fut_info)

        # Add options to instrument list
        for strike in range(min_strike, max_strike, interval):
            for exp_date in _get_expiry_list(symbol):
                ce_info = token_manager.get_opt(symbol, exp_date, strike, 'CE')
                pe_info = token_manager.get_opt(symbol, exp_date, strike, 'PE')
                if ce_info != None:
                    token_list.append(ce_info)
                if pe_info != None:
                    token_list.append(pe_info)

        return token_list
    else:
        raise "Unsupported instrument: {}".format(symbol)

if __name__ == '__main__':
    # Instruments test
    import logging
    logging.basicConfig(level=logging.DEBUG)
    exp_list = _get_expiry_list('NIFTY')
    d = get_derivatives('NIFTY')
    print(d)
    # Expiry date test
    exp_date = get_kth_expiry('FINNIFTY', type=ExpiryType.MONTHLY)
    print(exp_date)


import time

from SmartApi import SmartConnect
import mintotp
import requests
import logging

from alphatools.utils import cache

INSTRUMENT_API_URL = 'http://margincalculator.angelbroking.com/OpenAPI_File/files/OpenAPIScripMaster.json'
CACHE_FILE_PATH = '/tmp/'


def _candle_info_params_hash(candle_info_params):
    exch = candle_info_params["exchange"]
    tok = candle_info_params["symboltoken"]
    interval = candle_info_params["interval"]
    fromdate = candle_info_params["fromdate"]
    todate = candle_info_params["todate"]

    return "{}.{}.{}.{}.{}".format(exch, tok, interval, fromdate, todate)


class SmartApiHelper:
    logger = logging.getLogger(__name__)

    def __init__(self, api_key, client_code, password, totp_key):
        self.api_key = api_key
        self.client_code = client_code
        self.passwd = password
        self.totp_key = totp_key

    @staticmethod
    def get_instruments_list():
        instruments_csv_list = requests.get(INSTRUMENT_API_URL)
        return instruments_csv_list.json()

    def get_candle_info(self, candle_info_params):
        candle_info = self._get_candle_info(candle_info_params)
        if 'data' not in candle_info or not candle_info['data']:
            # print (candle_info['data'])
            # print("Could not load data from cache")
            return self._get_candle_info(candle_info_params, force_reload=True)
        
        return candle_info
    
    @cache.cache_to_file(CACHE_FILE_PATH, _candle_info_params_hash)
    # def _get_candle_info_wrap(self, candle_info_params):
    #     self.logger.info("Calling getCandleInfo wrapper")

    #     return self._get_candle_info(candle_info_params)

    def _get_candle_info(self, candle_info_params):
        candle_info_results = []

        n_retries_left = 3
        while n_retries_left > 0:
            try:
                smart_conn = SmartConnect(api_key=self.api_key)
                totp = mintotp.totp(self.totp_key)
                conn = smart_conn.generateSession(self.client_code, self.passwd, totp)
                refresh_token = conn['data']['refreshToken']
                candle_info_results = smart_conn.getCandleData(candle_info_params)
                smart_conn.terminateSession(self.client_code)
            except Exception as e:
                logging.exception("Historical API failed with exception")
                logging.info("Waiting for 5 seconds.")
                time.sleep(5)
                n_retries_left -= 1
                logging.info("Retrying. {} retries left".format(n_retries_left))

            return candle_info_results
